<?php
    /*
    * Plugin Name: Delyva WooCommerce Shipping
    * Plugin URI: https://delyva.com/
    * Version: 1.0.1
    * Description: A plugin to automate shipping and fulfilment via Delyva.com. Requires WooCommerce Version: 1.0
    * Author: Delyva.com
    * Author URI: https://delyva.com
    * License: GPL2+
    * License URI: http://www.gnu.org/licenses/gpl-2.0.txt
    */

    // Include functions.php, use require_once to stop the script if functions.php is not found
    defined( 'ABSPATH' ) or die( 'No script kiddies please!' );
    define('DELYVA_PLUGIN_VERSION', '1.0.1');

    require_once plugin_dir_path(__FILE__) . 'functions.php';
    // require_once plugin_dir_path(__FILE__) . 'includes/shipping-widget.php'; // TEMPORARILY DISABLED

    if ( in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {
        if ( ! class_exists( 'WC_Integration_Delyva' ) ) {
            class WC_Integration_Delyva {
                public function __construct() {
                    add_action('woocommerce_shipping_init', array( $this, 'init' ) );
                    add_filter('plugin_action_links_' . plugin_basename(__FILE__), array($this, 'plugin_action_links'));
                }

                public function init() {
                    add_filter('woocommerce_shipping_methods', array($this, 'add_shipping_method'));
                    if (class_exists('WC_Integration')) {
                        // Include our integration class.
                        include_once 'includes/delyva-shipping.php';
                        // Register the integration.
                        add_filter('woocommerce_shipping_methods', array($this, 'add_integration'));
                    }
                }

                public function plugin_action_links($links) {
                    return array_merge(
                        $links,
                        array('<a href="' . admin_url('admin.php?page=wc-settings&tab=shipping&section=delyva') . '"> ' . __('Settings', 'delyva') . '</a>')
                    );
                }

                public function add_shipping_method($methods) {
                    if (is_array($methods)) {
                        $methods['delyva'] = 'Delyva_Shipping_Method';
                    }
                    return $methods;
                }

                public function add_integration($integrations) {
                    $integrations[] = 'Delyva_Shipping_Method';
                    return $integrations;
                }
            }
            $WC_Integration_Delyva = new WC_Integration_Delyva();
        }
    }
