<?php
defined( 'ABSPATH' ) or die( 'No script kiddies please!' );

if (!class_exists('Delyva_Shipping_Method')) {

  class Delyva_Shipping_Method extends WC_Shipping_Method {

      /**
       * Constructor for your shipping class.
       */
      public function __construct($instance_id = 0) {
          $this->id = 'delyva';
          $this->instance_id = absint($instance_id);
          $this->method_title = __('Delyva.com', 'delyva');  // Title shown in admin
          $settings = get_option( 'woocommerce_delyva_settings' );
          if ($settings['integration_id']) {
              $this->method_description = __('<a href="https://www.delyva.com" target="_blank">Delyva.com</a> dynamic shipping rates at checkout.', 'delyva');
          } else {
              $this->method_description = __('<h3 style="color:red"><b>NOTICE!</b> This app is not configured! Please log in to Delyva.com then navigate to "Integration > Setup"</h3>', 'delyva');
          }
          $this->supports = array(
              'shipping-zones',
              'settings'
          );
          $this->init();
          $this->enabled = isset($this->settings['enabled']) ? $this->settings['enabled'] : 'yes';
          $this->title = isset($this->settings['title']) ? $this->settings['title'] : __('Delyva.com', 'delyva');
          add_action('woocommerce_update_options_shipping_' . $this->id, array($this, 'process_admin_options'));
      }

      /**
       * Init your settings.
       */
      public function init() {
          // Load the settings API
          $this->init_form_fields(); // This is part of the settings API. Override the method to add your own settings
          $this->init_settings(); // This is part of the settings API. Loads settings you previously init.
          add_filter('woocommerce_settings_tabs_array', __CLASS__ . '::add_settings_tab', 50);
          // Save settings in admin if you have any defined
          add_action('woocommerce_update_options_shipping_' . $this->id, array($this, 'process_admin_options'));
      }

      public static function add_settings_tab($settings_tabs) {
          $settings_tabs['shipping&section=delyva'] = __('Delyva', 'delyva-shipping');
          return $settings_tabs;
      }

      public function init_form_fields() {
          $this->form_fields = array(
            array(
                'title' => __( 'Settings', 'delyva' ),
                'type' => 'title',
                'id' => 'delyva_settings_title',
            ),
            'enable' => array(
                'title'    	=> __( 'Shipping Rate', 'delyva' ),
                'id'       	=> 'delyva_pricing_enable',
                'description'  	=> __( 'Enable dynamic shipping rate on checkout', 'delyva' ),
                'type'     	=> 'checkbox',
                'default'	=> 'yes'
            ),
            'api_key' => array(
                'title' => __('API Key', 'delyva'),
                'type' => 'text',
                'default' => __('', 'delyva'),
                'id' => 'delyva_api_key',
                'description' => __( 'Delyva API Key' ),
            ),
            'integration_id' => array(
                'title' => __('Integration ID', 'delyva'),
                'type' => 'text',
                'default' => __('', 'delyva'),
                'id' => 'delyva_integration_id',
            ),
              array(
                  'title' => __( 'Shipping Rate Adjustments', 'delyva' ),
                  'type' => 'title',
                  'id' => 'wc_settings_delyva_shipping_rate_adjustment',
                  'description' => __( 'Formula, shipping cost = shipping price + % rate + flat rate' ),
              ),
              'rate_adjustment_percentage' => array(
                  'title' => __('Percentage Rate %', 'delyva'),
                  'type' => 'text',
                  'default' => __('0', 'delyva'),
                  'id' => 'delyva_rate_adjustment_percentage'
              ),
              'rate_adjustment_flat' => array(
                  'title' => __('Flat Rate', 'delyva'),
                  'type' => 'text',
                  'default' => __('0', 'delyva'),
                  'id' => 'delyva_rate_adjustment_flat'
              ),
          );
      }

      /**
       * calculate_shipping function.
       *
       * @param mixed $package
       */
      public function calculate_shipping($package = array())
      {
            $destination = $package["destination"];
            $items = array();
            $product_factory = new WC_Product_Factory();
            $currency = get_woocommerce_currency();

            if (defined('WOOCS_VERSION')) {
                $currency = get_option('woocs_welcome_currency');
            }
            if (method_exists(WC()->cart, 'get_discount_total')) {
                $total_discount = WC()->cart->get_discount_total();
            } elseif (method_exists(WC()->cart, 'get_cart_discount_total')) {
                $total_discount = WC()->cart->get_cart_discount_total();
            } else {
                $total_discount = 0;
            }
            if (method_exists(WC()->cart, 'get_subtotal')) {
                $total_cart_without_discount = WC()->cart->get_subtotal();
            } else {
                $total_cart_without_discount = WC()->cart->subtotal;
            }
            if (!empty($total_discount) && ($total_discount > 0)) {
                $discount_for_item = ($total_discount / $total_cart_without_discount) * 100;
                $this->setDiscountForItem($discount_for_item);
                unset($discount_for_item);
            }
            foreach ($package["contents"] as $key => $item) {
                $product = $product_factory->get_product($item["product_id"]);
                $skip_shipping_class = $this->get_option('skip_shipping_class');
                if (!empty($skip_shipping_class) && ($product->get_shipping_class() === $skip_shipping_class)) {
                    continue;
                }
                if (WC()->version < '2.7.0') {
                    // if this item is variation, get variation product instead
                    if ($item["data"]->product_type == "variation") {
                        $product = $product_factory->get_product($item["variation_id"]);
                    }
                    // exclude virtual and downloadable product
                    if ($item["data"]->virtual == "yes") {
                        continue;
                    }
                } else {
                    if ($item["data"]->get_type() == "variation") {
                        $product = $product_factory->get_product($item["variation_id"]);
                    }
                    if ($item["data"]->get_virtual() == "yes") {
                        continue;
                    }
                }

                for ($i = 0; $i < $item["quantity"]; $i++) {
                    $items[] = array(
                        "actual_weight" => $this->weightToKg($product->get_weight()),
                        "height" => $this->defaultDimension($this->dimensionToCm($product->get_height())),
                        "width" => $this->defaultDimension($this->dimensionToCm($product->get_width())),
                        "length" => $this->defaultDimension($this->dimensionToCm($product->get_length())),
                        "declared_currency" => $currency,
                        "declared_customs_value" => $this->declaredCustomsValue($item['line_subtotal'], $item['quantity']),
                        "identifier_id" => array_key_exists("variation_id", $item) ? ($item["variation_id"] == 0 ? $item["product_id"] : $item["variation_id"]) : $item["product_id"]
                    );
                }
            }
            if (method_exists(WC()->cart, 'get_cart_contents_total')) {
                $total_cart_with_discount = (float)WC()->cart->get_cart_contents_total();
            } else {
                $total_cart_with_discount = WC()->cart->cart_contents_total;
            }
            if ($this->control_discount != $total_cart_with_discount) {
                if (is_array($items) && isset($items[0]) && isset($items[0]['declared_customs_value'])) {
                    $diff = round(($total_cart_with_discount - $this->control_discount), 2);
                    $items[0]['declared_customs_value'] += $diff;
                    $this->addControlDiscount($diff);
                    unset($diff);
                }
            }

            if (!class_exists('Delyva_Shipping_API')) {
                include_once 'delyva-api.php';
            }

            try {
                $rates = Delyva_Shipping_API::getShippingRate($destination, $items);
            } catch (Exception $e) {
                $rates = array();
            }

            foreach ($rates as $shipper) {
                if (isset($shipper['service_name'])) {
                    $settings = get_option( 'woocommerce_delyva_settings' );
                    $ra_percentage = $settings['rate_adjustment_percentage'] ?? 1;
                    $percentRate = $ra_percentage / 100 * $shipper['total_price'];
                    $flatRate = $settings['rate_adjustment_flat'] ?? 0;

                    $rate = array(
                        'id' => $shipper['service_code'],
                        'label' => $shipper['service_name'],
                        'cost' => round($shipper['total_price'] + $percentRate + $flatRate, 2),
                        'taxes' => 'false',
                        'calc_tax' => 'per_order',
                        'meta_data' => array(
                            'service_code' => $shipper['service_code'],
                        ),
                    );
                    // Register the rate
                    wp_cache_add('delyva' . $rate["id"], $rate);
                    $this->add_rate($rate);
                }
            }
        }

      protected function weightToKg($weight)
        {
            $weight_unit = get_option('woocommerce_weight_unit');
            // convert other unit into kg
            if ($weight_unit != 'kg') {
                if ($weight_unit == 'g') {
                    return $weight * 0.001;
                } else if ($weight_unit == 'lbs') {
                    return $weight * 0.453592;
                } else if ($weight_unit == 'oz') {
                    return $weight * 0.0283495;
                }
            }
            // already kg
            return $weight;
        }

        protected function dimensionToCm($length)
        {
            $dimension_unit = get_option('woocommerce_dimension_unit');
            // convert other units into cm
            if ($dimension_unit != 'cm') {
                if ($dimension_unit == 'm') {
                    return $length * 100;
                } else if ($dimension_unit == 'mm') {
                    return $length * 0.1;
                } else if ($dimension_unit == 'in') {
                    return $length * 2.54;
                } else if ($dimension_unit == 'yd') {
                    return $length * 91.44;
                }
            }
            // already in cm
            return $length;
        }

        protected function defaultDimension($length)
        {
            // default dimension to 1 if it is 0
            return $length > 0 ? $length : 1;
        }

        protected function setDiscountForItem($count)
        {
            $this->discount_for_item = $count;
        }
        protected function getDiscountForItem()
        {
            return $this->discount_for_item;
        }
        protected function addControlDiscount($val)
        {
            $this->control_discount += $val;
        }

        protected function declaredCustomsValue($subtotal, $count)
        {
            $price = (float)(($subtotal / $count) * ((100 - $this->getDiscountForItem()) / 100));
            $price = round($price, 2);
            $this->addControlDiscount($price);
            return $price;
        }
  }
}
